from .components.designator import Designator
from .components.part_number import PartNumber
from .components.pin import Pin
from .components.rectangle import Rectangle
from .components.symbol import Symbol


pin_spacing = 50
pin_font_size = 40
pin_desc_spacing = 15


def calculate_height(symbol_data):
    left_height = len(symbol_data['left_side'])
    right_height = len(symbol_data['right_side'])
    return max(left_height, right_height) * pin_spacing + pin_spacing


def calculate_width(symbol_data):
    longest_text_left = 0
    for text in symbol_data['left_side']:
        if text != "!@#SEPARATOR#@!":
            longest_text_left = max(longest_text_left, len(text))

    longest_text_right = 0
    for text in symbol_data['right_side']:
        if text != "!@#SEPARATOR#@!":
            longest_text_right = max(longest_text_right, len(text))
    width = (longest_text_left + longest_text_right) * pin_font_size * 0.6 + 100
    return 50 * round(width / 50)


def rectangular_symbol_generator(component_data, generator_data):
    symbol = Symbol(part_number=component_data['part'],
                    designator=component_data['designator'])

    height = calculate_height(generator_data)
    width = calculate_width(generator_data)

    symbol.width = width + 2 * 100 + 40
    symbol.height = height + + 120

    body_x = -width/2
    body_y = -height/2

    symbol.designator = Designator(designator=component_data['designator'], x=body_x, y=body_y - 10)
    symbol.part_number = PartNumber(text=component_data['part'], x=body_x, y=body_y + height + 40)
    symbol.add_body(Rectangle(width=width, height=height, x=body_x, y=body_y))

    add_pins(symbol, component_data, generator_data, width, body_y)

    return symbol


def add_pins(symbol, component_data, generator_data, width, body_y):
    left_pin_begin = -width / 2 - 100
    for i, left_pin in enumerate(generator_data['left_side']):
        if left_pin != "!@#SEPARATOR#@!":
            pin_data = component_data['pins'][left_pin]
            symbol.add_pin(Pin(name=left_pin,
                               number=pin_data['no'],
                               function=pin_data['func'],
                               description=pin_data['desc'] if 'desc' in pin_data else None,
                               x=int(left_pin_begin),
                               y=int(body_y + pin_spacing + i * pin_spacing),
                               length=100,
                               rotation=0))

    right_pin_begin = width / 2 + 100
    for i, right_pin in enumerate(generator_data['right_side']):
        if right_pin != "!@#SEPARATOR#@!":
            pin_data = component_data['pins'][right_pin]
            symbol.add_pin(Pin(name=right_pin,
                               number=pin_data['no'],
                               function=pin_data['func'],
                               description=pin_data['desc'] if 'desc' in pin_data else None,
                               x=int(right_pin_begin),
                               y=int(body_y + pin_spacing + i * pin_spacing),
                               length=100,
                               rotation=180))



import json

from pathlib import Path
from src.part_library_gen.symbol_generator import generate


def load_files(directory):
    files = Path(directory).rglob('*.json')
    return [x for x in files if not str(x).endswith('_generated.json')]


def generate_schematic_symbols(files):
    for file in files:
        with open(file, 'r') as f:
            data = json.load(f)
            for part in data:
                generate(part)


if __name__ == "__main__":
    generate_schematic_symbols(load_files("tests"))



class Pin:
    def __init__(self, name, number, function, description: str, x: int, y: int, length: int, rotation: int):
        self.name = name
        self.name_visible = True
        self.number = number
        self.number_visible = True
        self.function = function
        self.description = description
        self.x = x
        self.y = y
        self.length = length
        self.rotation = rotation





import json
import argparse

from .symbol_generator import generate


def main():
    generate_schematic_symbols()


def generate_schematic_symbols(files):
    for file in files:
        with open(file, 'r') as f:
            data = json.load(f)
            for part in data:
                generate(part)


if __name__ == '__main__':
    main()



import unittest
from src.part_library_gen.generators.rectangular_symbol_generator import rectangular_symbol_generator


class TestRectangularComponentGenerator(unittest.TestCase):
    def test_symbol_generation(self):
        symbol_dict = {
            "designator": "U?",
            "manufacturer": "Test Manufacturer Name",
            "part": "Test Part Name",
            "pins": {
                "I": {"no": 1, "func": "In", "desc": "ddl"},
                "O": {"no": 2, "func": "Out", "desc": "ddl"},
                "IA": {"no": 3, "func": "InAnalog", "desc": "ddl"},
                "OA": {"no": 4, "func": "OutAnalog", "desc": "ddl"},
                "ID": {"no": 5, "func": "InDigital", "desc": "ddl"},
                "OD": {"no": 6, "func": "OutDigital", "desc": "ddl"},
                "PI": {"no": 7, "func": "PwrIn", "desc": "ddl"},
                "PO": {"no": 8, "func": "PwrOut", "desc": "ddl"},
                "PG": {"no": 9, "func": "PwrGND", "desc": "ddl"}
            }
        }
        generator_data = {
            "left_side": ["I", "O", "IA", "OA", "ID"],
            "right_side": ["OD", "PI", "PO", "PG"]
        }
        generated_symbol = rectangular_symbol_generator(symbol_dict, generator_data)

        self.assertEqual(generated_symbol.designator.designator, "U?")
        self.assertEqual(generated_symbol.part_number.text, "Test Part Name")
        self.assertEqual(len(generated_symbol.pins), 9)

        self.assertEqual(generated_symbol.pins[0].name, 'I')
        self.assertEqual(generated_symbol.pins[1].name, 'O')
        self.assertEqual(generated_symbol.pins[2].name, 'IA')
        self.assertEqual(generated_symbol.pins[3].name, 'OA')

        self.assertEqual(generated_symbol.pins[0].number, 1)
        self.assertEqual(generated_symbol.pins[1].number, 2)
        self.assertEqual(generated_symbol.pins[2].number, 3)
        self.assertEqual(generated_symbol.pins[3].number, 4)

        self.assertEqual(generated_symbol.pins[0].function, 'In')
        self.assertEqual(generated_symbol.pins[1].function, 'Out')
        self.assertEqual(generated_symbol.pins[2].function, 'InAnalog')
        self.assertEqual(generated_symbol.pins[3].function, 'OutAnalog')



import drawsvg as svg

pin_spacing = 50
pin_font_size = 40
pin_desc_spacing = 15

in_arrow = svg.Marker(-1.5, -0.61, 0, 0.6, scale=4, orient='auto')
in_arrow.append(svg.Lines(-1, 0.5, -1, -0.5, 0, 0, stroke_width=0.1, close=True, fill='gray', stroke='black'))

out_arrow = svg.Marker(-1.5, -0.61, 0, 0.6, scale=4, orient='auto')
out_arrow.append(svg.Lines(-1, 0, 0, -0.5, 0, 0.5, stroke_width=0.1, close=True, fill='gray', stroke='black'))

bidirectional_arrow = svg.Marker(-2.3, -0.61, 0, 0.6, scale=4, orient='auto')
bidirectional_arrow.append(svg.Lines(-1, 0.5, -1, -0.5, 0, 0, stroke_width=0.1, close=True, fill='gray', stroke='black'))
bidirectional_arrow.append(svg.Lines(-2.3, 0, -1.3, -0.5, -1.3, 0.5, stroke_width=0.1, close=True, fill='gray', stroke='black'))

marker_map = {"In": in_arrow,
              "InAnalog": in_arrow,
              "InDigital": in_arrow,
              "Out": out_arrow,
              "OutAnalog": out_arrow,
              "OutDigital": out_arrow,
              "InOut": bidirectional_arrow}


def generate_symbol_pin(pin):
    group = svg.Group()
    if pin.rotation == 0:
        pin_end = pin.x + pin.length
        group.append(svg.Line(pin.x,
                              pin.y,
                              pin_end,
                              pin.y,
                              stroke_width=5,
                              stroke='black',
                              marker_end=marker_map[pin.function]))
        if pin.name:
            group.append(svg.Text(pin.name,
                                  pin_font_size,
                                  pin_end + pin_desc_spacing,
                                  pin.y + pin_font_size / 4))
        if pin.number:
            if isinstance(pin.number, list) and len(pin.number) == 1:
                pin_no_str = str(pin.number[0])
            else:
                pin_no_str = str(pin.number)
            group.append(svg.Text(pin_no_str,
                                  pin_font_size,
                                  pin_end - 40,
                                  pin.y - 5,
                                  text_anchor='end'))
    elif pin.rotation == 180:
        pin_end = pin.x - pin.length
        group.append(svg.Line(pin.x ,
                              pin.y,
                              pin.x- pin.length,
                              pin.y,
                              stroke_width=5,
                              stroke='black',
                              marker_end=marker_map[pin.function]))
        if pin.name:
            group.append(svg.Text(pin.name,
                                  pin_font_size,
                                  pin_end - pin_desc_spacing,
                                  pin.y + pin_font_size / 4,
                                  text_anchor='end'))
        if pin.number:
            if isinstance(pin.number, list) and len(pin.number) == 1:
                pin_no_str = str(pin.number[0])
            else:
                pin_no_str = str(pin.number)
            group.append(svg.Text(pin_no_str,
                                  pin_font_size,
                                  pin_end + 40,
                                  pin.y - 5))
    return group



from .generators.rectangular_symbol_generator import rectangular_symbol_generator
from .generators.default_generator import default_generator
from .exporters.svg.svg_exporter import export as svg_exporter

supported_pin_functions = ['In', 'Out', 'InAnalog', 'InDigital',
                           'OutDigital', 'OutAnalog',
                           'InOut',
                           'PwrIn', 'PwrOut', 'PwrGND']


def validate(component_data):
    if component_data is not None:
        if 'designator' not in component_data:
            raise ValueError('designator is missing')
        if 'part' not in component_data:
            raise ValueError('part number is missing')
        if 'pins' not in component_data:
            raise ValueError('pins are missing')
        for pin in component_data['pins']:
            pin_data = component_data['pins'][pin]

            if 'func' not in pin_data:
                raise ValueError('pin function is missing')
            if 'no' not in pin_data:
                raise ValueError('pin number is missing')
            if pin_data['func'] not in supported_pin_functions:
                raise ValueError('pin function is not supported: {}'.format(pin_data['func']))


def generate_file_name(component_data, generator_name):
    manufacturer_name = component_data['manufacturer'].replace(' ', '_')
    part = component_data['part'].replace('#', '')
    return f"symbols/{manufacturer_name}_{part}_{generator_name}"


def generate(data):
    generator_map = {'default': default_generator,
                     'rectangle': rectangular_symbol_generator}

    validate(data)
    if 'symbol_generator' not in data:
        symbol = default_generator(data, {})
        export_symbol(symbol, generate_file_name(data, 'generic'))
    else:
        for generator in data['symbol_generator']:
            generator_data = data['symbol_generator'][generator]
            filename = generate_file_name(data, generator)
            symbol = generator_map[generator](data, generator_data)
            export_symbol(symbol, filename)


def export_symbol(symbol, filename):
    exporters = [svg_exporter]
    for exporter in exporters:
        exporter(symbol, filename)



class Symbol:
    def __init__(self, part_number, designator):
        self.part_number = part_number
        self.designator = designator
        self.pins = []
        self.body = []
        self.width = 0
        self.height = 0

    def add_pin(self, pin):
        self.pins.append(pin)

    def add_body(self, element):
        self.body.append(element)


import drawsvg as svg
from .pin import generate_symbol_pin
from src.part_library_gen.generators.components.rectangle import Rectangle


def export(symbol, filename):
    d = svg.Drawing(symbol.width, symbol.height, origin='center')

    for element in symbol.body:
        if isinstance(element, Rectangle):
            d.append(svg.Rectangle(element.x,
                                   element.y,
                                   element.width,
                                   element.height,
                                   stroke_width=3,
                                   fill='yellow',
                                   stroke='black'))

    for pin in symbol.pins:
        d.append(generate_symbol_pin(pin))

    d.append(svg.Text(symbol.designator.designator,
                      40,
                      symbol.designator.x,
                      symbol.designator.y))

    d.append(svg.Text(symbol.part_number.text,
                      40,
                      symbol.part_number.x,
                      symbol.part_number.y))

    d.save_svg(f"{filename}.svg")



from .rectangular_symbol_generator import rectangular_symbol_generator


def default_generator(data, generator_data):
    half = len(data['pins']) // 2

    left_side = data['pins'].keys()
    generator_data['left_side'] = list(left_side)[:half]
    generator_data['right_side'] = list(left_side)[half:]
    return rectangular_symbol_generator(data, generator_data)
